'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var path = require('path');

var CircularJSON = require('circular-json');
var Debug = require('debug');
var fastJson = require('fast-json-stringify');
// const jsonfile = require('jsonfile');
var mkdirp = require('mkdirp');
var Vue = require('vue');

var asyncDebounce = require('./asyncDebounce');
var jsonfile = require('./jsonfile');

var debug = Debug('bolt-json-db:Data');

module.exports = function (JSONPath, initData, Schema) {
    var parse = void 0;
    var stringify = void 0;
    if (Schema) {
        parse = JSON.parse;
        stringify = fastJson(Schema);
    } else {
        parse = CircularJSON.parse;
        stringify = CircularJSON.stringify;
    }

    var JSONDir = path.dirname(JSONPath);
    var jsonfileWriteFileDebounce = asyncDebounce(function (args, next) {
        mkdirp(JSONDir);
        jsonfile.writeFile(args[0], args[1], {
            parse,
            stringify
        }, function (err) {
            args[2](err);
            next();
        });
    });

    var data = void 0;
    try {
        data = jsonfile.readFileSync(JSONPath, {
            parse,
            stringify
        });
    } catch (e) {
        debug(`没有发现 ${JSONPath}, 文件改变时将会自动创建`);
    };

    var store = new Vue({
        data: {
            data: _extends({}, initData, data)
        },
        watch: {
            data: {
                handler: function handler(val) {
                    jsonfileWriteFileDebounce(JSONPath, val, function (err) {
                        if (err) {
                            console.error(err);
                        }
                    });
                },
                deep: true
            }
        }
    });

    return store.data;
};